@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 import collections.abc
+import inspect
+import logging
 from collections import OrderedDict, namedtuple
 from typing import TYPE_CHECKING, Any, Union
 
@@ -18,6 +20,8 @@ from .utils import (
     hist_object_handler,
     isLight,
     process_histogram_parts,
+    align_marker,
+    to_padded2d,
 )
 
 if TYPE_CHECKING:
@@ -27,7 +31,6 @@ StairsArtists = namedtuple("StairsArtists", "stairs errorbar legend_artist")
 ErrorBarArtists = namedtuple("ErrorBarArtists", "errorbar")
 ColormeshArtists = namedtuple("ColormeshArtists", "pcolormesh cbar text")
 
-
 Hist1DArtists = Union[StairsArtists, ErrorBarArtists]
 Hist2DArtists = ColormeshArtists
 
@@ -55,7 +58,7 @@ def soft_update_kwargs(kwargs, mods, rc=True):
 # Histogram plotter
 def histplot(
     H,  # Histogram object, tuple or array
-    bins=None,  # Bins to be supplied when h is a value array or iterable of arrays
+    bins=None,  # Bins to be supplied when h is a value array or iterable of array
     *,
     yerr: ArrayLike | bool | None = None,
     w2=None,
@@ -70,6 +73,7 @@ def histplot(
     edges=True,
     binticks=False,
     ax=None,
+    flow="hint",
     **kwargs,
 ):
     """
@@ -87,7 +91,7 @@ def histplot(
 
             Or list thereof.
         bins : iterable, optional
-            Histogram bins, if not part of ``h``.
+            Histogram bins, if not part of ``H``.
         yerr : iterable or bool, optional
             Histogram uncertainties. Following modes are supported:
             - True, sqrt(N) errors or poissonian interval when ``w2`` is specified
@@ -112,12 +116,13 @@ def histplot(
         binwnorm : float, optional
             If true, convert sum weights to bin-width-normalized, with unit equal to
                 supplied value (usually you want to specify 1.)
-        histtype: {'step', 'fill', 'errorbar'}, optional, default: "step"
+        histtype: {'step', 'fill', 'band', 'errorbar'}, optional, default: "step"
             Type of histogram to plot:
 
-            - "step": skyline/step/outline of a histogram using `plt.step <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.axes.Axes.step.html#matplotlib.axes.Axes.step>`_
-            - "fill": filled histogram using `plt.fill_between <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.axes.Axes.step.html#matplotlib.axes.Axes.step>`_
-            - "errorbar": single marker histogram using `plt.errorbar <https://matplotlib.org/3.1.1/api/_as_gen/matplotlib.axes.Axes.step.html#matplotlib.axes.Axes.step>`_
+            - "step": skyline/step/outline of a histogram using `plt.stairs <https://matplotlib.org/stable/api/_as_gen/matplotlib.axes.Axes.stairs.html#matplotlib-axes-axes-stairs>`_
+            - "fill": filled histogram using `plt.stairs <https://matplotlib.org/stable/api/_as_gen/matplotlib.axes.Axes.stairs.html#matplotlib-axes-axes-stairs>`_
+            - "step": filled band spanning the yerr range of the histogram using `plt.stairs <https://matplotlib.org/stable/api/_as_gen/matplotlib.axes.Axes.stairs.html#matplotlib-axes-axes-stairs>`_
+            - "errorbar": single marker histogram using `plt.errorbar <https://matplotlib.org/stable/api/_as_gen/matplotlib.axes.Axes.errorbar.html#matplotlib-axes-axes-errorbar>`_
         xerr:  bool or float, optional
             Size of xerr if ``histtype == 'errorbar'``. If ``True``, bin-width will be used.
         label : str or list, optional
@@ -130,14 +135,18 @@ def histplot(
             Attempts to draw x-axis ticks coinciding with bin boundaries if feasible.
         ax : matplotlib.axes.Axes, optional
             Axes object (if None, last one is fetched or one is created)
+        flow :  str, optional { "show", "sum", "hint", "none"}
+            Whether plot the under/overflow bin. If "show", add additional under/overflow bin.
+            If "sum", add the under/overflow bin content to first/last bin.
         **kwargs :
             Keyword arguments passed to underlying matplotlib functions -
-            {'step', 'fill_between', 'errorbar'}.
+            {'stairs', 'errorbar'}.
     Returns
     -------
         List[Hist1DArtists]
 
     """
+
     # ax check
     if ax is None:
         ax = plt.gca()
@@ -146,9 +155,15 @@ def histplot(
             raise ValueError("ax must be a matplotlib Axes object")
 
     # arg check
-    _allowed_histtype = ["fill", "step", "errorbar"]
+    _allowed_histtype = ["fill", "step", "errorbar", "band"]
     _err_message = f"Select 'histtype' from: {_allowed_histtype}"
     assert histtype in _allowed_histtype, _err_message
+    assert flow is None or flow in {
+        "show",
+        "sum",
+        "hint",
+        "none",
+    }, "flow must be show, sum, hint, or none"
 
     # Convert 1/0 etc to real bools
     stack = bool(stack)
@@ -159,6 +174,8 @@ def histplot(
     # Process input
     hists = list(process_histogram_parts(H, bins))
     final_bins, xtick_labels = get_plottable_protocol_bins(hists[0].axes[0])
+    _bin_widths = np.diff(final_bins)
+    _bin_centers = final_bins[1:] - _bin_widths / float(2)
     assert final_bins.ndim == 1, "bins need to be 1 dimensional"
     _x_axes_label = ax.get_xlabel()
     x_axes_label = (
@@ -167,10 +184,77 @@ def histplot(
         else get_histogram_axes_title(hists[0].axes[0])
     )
 
-    # Cast to plottables
-    plottables = [
-        Plottable(h.values(), edges=final_bins, variances=h.variances()) for h in hists
-    ]
+    plottables = []
+    flow_bins = final_bins
+    for i, h in enumerate(hists):
+        value, variance = np.copy(h.values()), h.variances()
+        if has_variances := variance is not None:
+            variance = np.copy(variance)
+        underflow, overflow = 0.0, 0.0
+        underflowv, overflowv = 0.0, 0.0
+        # One sided flow bins - hist (uproot hist does not have the over- or underflow traits)
+        if (
+            hasattr(h, "axes")
+            and (traits := getattr(h.axes[0], "traits", None)) is not None
+            and hasattr(traits, "underflow")
+            and hasattr(traits, "overflow")
+        ):
+            if traits.overflow:
+                overflow = np.copy(h.values(flow=True))[-1]
+                if has_variances:
+                    overflowv = np.copy(h.variances(flow=True))[-1]
+            if traits.underflow:
+                underflow = np.copy(h.values(flow=True))[0]
+                if has_variances:
+                    underflowv = np.copy(h.variances(flow=True))[0]
+        # Both flow bins exist - uproot
+        elif hasattr(h, "values") and "flow" in inspect.getfullargspec(h.values).args:
+            if len(h.values()) + 2 == len(
+                h.values(flow=True)
+            ):  # easy case, both over/under
+                underflow, overflow = (
+                    np.copy(h.values(flow=True))[0],
+                    np.copy(h.values(flow=True))[-1],
+                )
+                if has_variances:
+                    underflowv, overflowv = (
+                        np.copy(h.variances(flow=True))[0],
+                        np.copy(h.variances(flow=True))[-1],
+                    )
+
+        # Set plottables
+        if flow == "none":
+            plottables.append(Plottable(value, edges=final_bins, variances=variance))
+        elif flow == "hint":  # modify plottable
+            plottables.append(Plottable(value, edges=final_bins, variances=variance))
+        elif flow == "show":
+            _flow_bin_size = np.max(
+                [0.05 * (final_bins[-1] - final_bins[0]), np.mean(np.diff(final_bins))]
+            )
+            flow_bins = np.copy(final_bins)
+            if underflow > 0:
+                flow_bins = np.r_[flow_bins[0] - _flow_bin_size, flow_bins]
+                value = np.r_[underflow, value]
+                if has_variances:
+                    variance = np.r_[underflowv, variance]
+            if overflow > 0:
+                flow_bins = np.r_[flow_bins, flow_bins[-1] + _flow_bin_size]
+                value = np.r_[value, overflow]
+                if has_variances:
+                    variance = np.r_[variance, overflowv]
+            plottables.append(Plottable(value, edges=flow_bins, variances=variance))
+        elif flow == "sum":
+            if underflow > 0:
+                value[0] += underflow
+                if has_variances:
+                    variance[0] += underflowv
+            if overflow > 0:
+                value[-1] += overflow
+                if has_variances:
+                    variance[-1] += overflowv
+            plottables.append(Plottable(value, edges=final_bins, variances=variance))
+        else:
+            plottables.append(Plottable(value, edges=final_bins, variances=variance))
 
     if w2 is not None:
         for _w2, _plottable in zip(
@@ -201,8 +285,10 @@ def histplot(
     for kwarg in kwargs:
         # Check if iterable
         if iterable_not_string(kwargs[kwarg]):
-            # Check if tuple (can be used for colors)
-            if type(kwargs[kwarg]) == tuple:
+            # Check if tuple of floats or ints (can be used for colors)
+            if isinstance(kwargs[kwarg], tuple) and all(
+                isinstance(x, int) or isinstance(x, float) for x in kwargs[kwarg]
+            ):
                 for i in range(len(_chunked_kwargs)):
                     _chunked_kwargs[i][kwarg] = kwargs[kwarg]
             else:
@@ -212,9 +298,6 @@ def histplot(
             for i in range(len(_chunked_kwargs)):
                 _chunked_kwargs[i][kwarg] = kwargs[kwarg]
 
-    _bin_widths = np.diff(final_bins)
-    _bin_centers = final_bins[1:] - _bin_widths / float(2)
-
     ############################
     # # yerr calculation
     _yerr: np.ndarray | None
@@ -272,7 +355,7 @@ def histplot(
         elif isinstance(sort, list) or isinstance(sort, np.ndarray):
             if len(sort) != len(plottables):
                 raise ValueError(
-                    f"Sort indexing arrays is of the wrong size - {len(sort)}, {len(plottables)} expected."
+                    f"Sort indexing array is of the wrong size - {len(sort)}, {len(plottables)} expected."
                 )
             order = np.asarray(sort)
         else:
@@ -299,7 +382,8 @@ def histplot(
         for plottable, norm in zip(
             plottables, np.broadcast_to(binwnorm, (len(plottables),))
         ):
-            plottable.flat_scale(norm / np.diff(final_bins))
+            plottable.flat_scale(norm)
+            plottable.binwnorm()
 
     # Stack
     if stack and len(plottables) > 1:
@@ -359,6 +443,24 @@ def histplot(
             return_artists.append(StairsArtists(_f, None, None))
         _artist = _f
 
+    elif histtype == "band":
+        band_defaults = {
+            "alpha": 0.5,
+            "edgecolor": "darkgray",
+            "facecolor": "whitesmoke",
+            "hatch": "////  /",
+        }
+        for i in range(len(plottables)):
+            _kwargs = _chunked_kwargs[i]
+            _f = ax.stairs(
+                **plottables[i].to_stairband(),
+                label=_labels[i],
+                fill=True,
+                **soft_update_kwargs(_kwargs, band_defaults),
+            )
+            return_artists.append(StairsArtists(_f, None, None))
+        _artist = _f
+
     elif histtype == "errorbar":
         err_defaults = {
             "linestyle": "none",
@@ -377,6 +479,7 @@ def histplot(
             _xerr = None
 
         for i in range(len(plottables)):
+            _kwargs = _chunked_kwargs[i]
             _plot_info = plottables[i].to_errorbar()
             if yerr is False:
                 _plot_info["yerr"] = None
@@ -384,7 +487,7 @@ def histplot(
             _e = ax.errorbar(
                 **_plot_info,
                 label=_labels[i],
-                **soft_update_kwargs(_chunked_kwargs[i], err_defaults),
+                **soft_update_kwargs(_kwargs, err_defaults),
             )
             return_artists.append(ErrorBarArtists(_e))
 
@@ -393,7 +496,7 @@ def histplot(
     # Add sticky edges for autoscale
     _artist.sticky_edges.y.append(0)
 
-    if xtick_labels is None:
+    if xtick_labels is None or flow == "show":
         if binticks:
             _slice = int(round(float(len(final_bins)) / len(ax.get_xticks()))) + 1
             ax.set_xticks(final_bins[::_slice])
@@ -404,6 +507,102 @@ def histplot(
     if x_axes_label:
         ax.set_xlabel(x_axes_label)
 
+    # Flow extra styling
+    if flow == "hint":
+        _marker_size = (
+            30
+            * ax.get_window_extent()
+            .transformed(ax.figure.dpi_scale_trans.inverted())
+            .width
+        )
+        if underflow > 0.0:
+            ax.scatter(
+                final_bins[0],
+                0,
+                _marker_size,
+                marker=align_marker("<", halign="right"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.get_xaxis_transform(),
+            )
+        if overflow > 0.0:
+            ax.scatter(
+                final_bins[-1],
+                0,
+                _marker_size,
+                marker=align_marker(">", halign="left"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.get_xaxis_transform(),
+            )
+
+    elif flow == "show" and (underflow > 0.0 or overflow > 0.0):
+        xticks = ax.get_xticks().tolist()
+        lw = ax.spines["bottom"].get_linewidth()
+        _edges = plottables[0].edges
+        _centers = plottables[0].centers
+        _marker_size = (
+            20
+            * ax.get_window_extent()
+            .transformed(ax.figure.dpi_scale_trans.inverted())
+            .width
+        )
+        if underflow > 0.0:
+            xticks[0] = ""
+            xticks[1] = f"<{flow_bins[2]}"
+            ax.set_xticklabels(xticks)
+
+            ax.plot(
+                [_edges[0], _edges[1]],
+                [0, 0],
+                color="white",
+                zorder=5,
+                ls="--",
+                lw=lw,
+                transform=ax.get_xaxis_transform(),
+                clip_on=False,
+            )
+            ax.scatter(
+                _centers[0],
+                0,
+                _marker_size,
+                marker=align_marker("d", valign="center"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.get_xaxis_transform(),
+            )
+        if overflow > 0.0:
+            xticks[-1] = ""
+            xticks[-2] = f">{flow_bins[-3]}"
+            ax.set_xticklabels(xticks)
+            ax.plot(
+                [_edges[-2], _edges[-1]],
+                [0, 0],
+                color="white",
+                zorder=5,
+                ls="--",
+                lw=lw,
+                transform=ax.get_xaxis_transform(),
+                clip_on=False,
+            )
+            ax.scatter(
+                _centers[-1],
+                0,
+                _marker_size,
+                marker=align_marker("d", valign="center"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.get_xaxis_transform(),
+            )
+
     return return_artists
 
 
@@ -416,10 +615,11 @@ def hist2dplot(
     cbarsize="7%",
     cbarpad=0.2,
     cbarpos="right",
-    cbarextend=False,
+    cbarextend=True,
     cmin=None,
     cmax=None,
     ax=None,
+    flow="hint",
     **kwargs,
 ):
     """
@@ -460,6 +660,8 @@ def hist2dplot(
         Colorbar maximum.
     ax : matplotlib.axes.Axes, optional
         Axes object (if None, last one is fetched or one is created)
+    flow :  str, optional {"show", "sum","hint", None}
+            Whether plot the under/overflow bin. If "show", add additional under/overflow bin. If "sum", add the under/overflow bin content to first/last bin. "hint" would highlight the bins with under/overflow contents
     **kwargs :
         Keyword arguments passed to underlying matplotlib function - pcolormesh.
 
@@ -476,22 +678,77 @@ def hist2dplot(
         if not isinstance(ax, plt.Axes):
             raise ValueError("ax must be a matplotlib Axes object")
 
-    hist = hist_object_handler(H, xbins, ybins)
+    h = hist_object_handler(H, xbins, ybins)
 
     # TODO: use Histogram everywhere
-    H = hist.values()
-    xbins, xtick_labels = get_plottable_protocol_bins(hist.axes[0])
-    ybins, ytick_labels = get_plottable_protocol_bins(hist.axes[1])
+
+    H = np.copy(h.values())
+    xbins, xtick_labels = get_plottable_protocol_bins(h.axes[0])
+    ybins, ytick_labels = get_plottable_protocol_bins(h.axes[1])
+    # Show under/overflow bins
+    # "show": Add additional bin with 2 times bin width
+    if (
+        hasattr(h, "values")
+        and "flow" not in inspect.getfullargspec(h.values).args
+        and flow is not None
+    ):
+        print(
+            f"Warning: {type(h)} is not allowed to get flow bins, flow bin option set to None"
+        )
+        flow = None
+    elif flow in ["hint", "show"]:
+        xwidth, ywidth = (xbins[-1] - xbins[0]) * 0.05, (ybins[-1] - ybins[0]) * 0.05
+        pxbins = np.r_[xbins[0] - xwidth, xbins, xbins[-1] + xwidth]
+        pybins = np.r_[ybins[0] - ywidth, ybins, ybins[-1] + ywidth]
+        padded = to_padded2d(h)
+        hint_xlo, hint_xhi, hint_ylo, hint_yhi = True, True, True, True
+        if np.all(padded[0, :] == 0):
+            padded = padded[1:, :]
+            pxbins = pxbins[1:]
+            hint_xlo = False
+        if np.all(padded[-1, :] == 0):
+            padded = padded[:-1, :]
+            pxbins = pxbins[:-1]
+            hint_xhi = False
+        if np.all(padded[:, 0] == 0):
+            padded = padded[:, 1:]
+            pybins = pybins[1:]
+            hint_ylo = False
+        if np.all(padded[:, -1] == 0):
+            padded = padded[:, :-1]
+            pybins = pybins[:-1]
+            hint_yhi = False
+        if flow == "show":
+            H = padded
+            xbins, ybins = pxbins, pybins
+    elif flow == "sum":
+        H = np.copy(h.values())
+        # Sum borders
+        H[0], H[-1] = (
+            H[0] + h.values(flow=True)[0, 1:-1],
+            H[-1] + h.values(flow=True)[-1, 1:-1],
+        )
+        H[:, 0], H[:, -1] = (
+            H[:, 0] + h.values(flow=True)[1:-1, 0],
+            H[:, -1] + h.values(flow=True)[1:-1, -1],
+        )
+        # Sum corners to corners
+        H[0, 0], H[-1, -1], H[0, -1], H[-1, 0] = (
+            h.values(flow=True)[0, 0] + H[0, 0],
+            h.values(flow=True)[-1, -1] + H[-1, -1],
+            h.values(flow=True)[0, -1] + H[0, -1],
+            h.values(flow=True)[-1, 0] + H[-1, 0],
+        )
     xbin_centers = xbins[1:] - np.diff(xbins) / float(2)
     ybin_centers = ybins[1:] - np.diff(ybins) / float(2)
 
     _x_axes_label = ax.get_xlabel()
     x_axes_label = (
-        _x_axes_label if _x_axes_label != "" else get_histogram_axes_title(hist.axes[0])
+        _x_axes_label if _x_axes_label != "" else get_histogram_axes_title(h.axes[0])
     )
     _y_axes_label = ax.get_ylabel()
     y_axes_label = (
-        _y_axes_label if _y_axes_label != "" else get_histogram_axes_title(hist.axes[1])
+        _y_axes_label if _y_axes_label != "" else get_histogram_axes_title(h.axes[1])
     )
 
     H = H.T
@@ -504,7 +761,7 @@ def hist2dplot(
     X, Y = np.meshgrid(xbins, ybins)
 
     kwargs.setdefault("shading", "flat")
-    pc = ax.pcolormesh(X, Y, H, **kwargs)
+    pc = ax.pcolormesh(X, Y, H, vmin=cmin, vmax=cmax, **kwargs)
 
     if x_axes_label:
         ax.set_xlabel(x_axes_label)
@@ -536,6 +793,98 @@ def hist2dplot(
         cb_obj = None
 
     plt.sca(ax)
+    if flow == "show":
+        if hint_xlo:
+            ax.plot(
+                [xbins[1]] * 2,
+                [0, 1],
+                ls="--",
+                color="lightgrey",
+                clip_on=False,
+                transform=ax.get_xaxis_transform(),
+            )
+        if hint_xhi:
+            ax.plot(
+                [xbins[-2]] * 2,
+                [0, 1],
+                ls="--",
+                color="lightgrey",
+                clip_on=False,
+                transform=ax.get_xaxis_transform(),
+            )
+        if hint_ylo:
+            ax.plot(
+                [0, 1],
+                [ybins[1]] * 2,
+                ls="--",
+                color="lightgrey",
+                clip_on=False,
+                transform=ax.get_yaxis_transform(),
+            )
+        if hint_yhi:
+            ax.plot(
+                [0, 1],
+                [ybins[-2]] * 2,
+                ls="--",
+                color="lightgrey",
+                clip_on=False,
+                transform=ax.get_yaxis_transform(),
+            )
+    elif flow == "hint":
+        _marker_size = (
+            30
+            * ax.get_window_extent()
+            .transformed(ax.figure.dpi_scale_trans.inverted())
+            .width
+        )
+        if hint_xlo:
+            ax.scatter(
+                0,
+                0,
+                _marker_size,
+                marker=align_marker("<", halign="right", valign="bottom"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.transAxes,
+            )
+        if hint_xhi:
+            ax.scatter(
+                1,
+                0,
+                _marker_size,
+                marker=align_marker(">", halign="left"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.transAxes,
+            )
+        if hint_ylo:
+            ax.scatter(
+                0,
+                0,
+                _marker_size,
+                marker=align_marker("v", valign="top", halign="left"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.transAxes,
+            )
+        if hint_yhi:
+            ax.scatter(
+                0,
+                1,
+                _marker_size,
+                marker=align_marker("^", valign="bottom"),
+                edgecolor="black",
+                zorder=5,
+                clip_on=False,
+                facecolor="white",
+                transform=ax.transAxes,
+            )
 
     _labels: np.ndarray | None = None
     if isinstance(labels, bool):
@@ -579,6 +928,7 @@ def overlap(ax, bbox, get_vertices=False):
     """
     from matplotlib.lines import Line2D
     from matplotlib.patches import Patch, Rectangle
+    from matplotlib.text import Text
 
     # From
     # https://github.com/matplotlib/matplotlib/blob/08008d5cb4d1f27692e9aead9a76396adc8f0b19/lib/matplotlib/legend.py#L845
@@ -594,13 +944,17 @@ def overlap(ax, bbox, get_vertices=False):
 
     for handle in ax.patches:
         assert isinstance(handle, Patch)
-
         if isinstance(handle, Rectangle):
             transform = handle.get_data_transform()
             bboxes.append(handle.get_bbox().transformed(transform))
         else:
-            transform = handle.get_transform()
-            bboxes.append(handle.get_path().get_extents(transform))
+            if len(handle.get_path().vertices) == 0:
+                continue
+            lines.append(handle.get_path().interpolated(20))
+
+    for handle in ax.texts:
+        assert isinstance(handle, Text)
+        bboxes.append(handle.get_window_extent())
 
     # TODO Possibly other objects
 
@@ -621,6 +975,10 @@ def _draw_leg_bbox(ax):
     """
     fig = ax.figure
     leg = ax.get_legend()
+    if leg is None:
+        leg = [
+            c for c in ax.get_children() if isinstance(c, plt.matplotlib.legend.Legend)
+        ][0]
 
     fig.canvas.draw()
     return leg.get_frame().get_bbox()
@@ -631,9 +989,10 @@ def _draw_text_bbox(ax):
     Draw legend() and fetch it's bbox
     """
     fig = ax.figure
-    textboxes = [k for k in ax.get_children() if type(k) == AnchoredText]
+    textboxes = [k for k in ax.get_children() if isinstance(k, AnchoredText)]
+    fig.canvas.draw()
     if len(textboxes) > 1:
-        print("Warning: More than one textbox found")
+        logging.warning("More than one textbox found")
         for box in textboxes:
             if box.loc in [1, 2]:
                 bbox = box.get_tightbbox(fig.canvas.renderer)
@@ -643,30 +1002,63 @@ def _draw_text_bbox(ax):
     return bbox
 
 
-def yscale_legend(ax=None):
+def yscale_legend(ax=None, otol=0, soft_fail=False):
     """
-    Automatically scale y-axis up to fit in legend()
+    Automatically scale y-axis up to fit in legend().
+    Set `otol > 0` for less strict scaling.
+    Set `soft_fail=True` to return even if it could not fit the legend.
     """
     if ax is None:
         ax = plt.gca()
 
     scale_factor = 10 ** (1.05) if ax.get_yscale() == "log" else 1.05
-    while overlap(ax, _draw_leg_bbox(ax)) > 0:
+    max_scales = 0
+    while overlap(ax, _draw_leg_bbox(ax)) > otol:
+        logging.debug(
+            f"Legend overlap with other artists is {overlap(ax, _draw_leg_bbox(ax))}."
+        )
+        logging.info("Scaling y-axis by 5% to fit legend")
         ax.set_ylim(ax.get_ylim()[0], ax.get_ylim()[-1] * scale_factor)
         ax.figure.canvas.draw()
+        if max_scales > 10:
+            if not soft_fail:
+                raise RuntimeError(
+                    "Could not fit legend in 10 iterations, return anyway by passing `soft_fail=True`."
+                )
+            else:
+                logging.warning("Could not fit legend in 10 iterations")
+                break
+        max_scales += 1
     return ax
 
 
-def yscale_text(ax=None):
+def yscale_anchored_text(ax=None, otol=0, soft_fail=False):
     """
     Automatically scale y-axis up to fit AnchoredText
+    Set `otol > 0` for less strict scaling.
+    Set `soft_fail=True` to return even if it could not fit the AnchoredText.
     """
     if ax is None:
         ax = plt.gca()
 
-    while overlap(ax, _draw_text_bbox(ax)) > 0:
-        ax.set_ylim(ax.get_ylim()[0], ax.get_ylim()[-1] * 1.1)
+    scale_factor = 10 ** (1.05) if ax.get_yscale() == "log" else 1.05
+    max_scales = 0
+    while overlap(ax, _draw_text_bbox(ax)) > otol:
+        logging.debug(
+            f"AnchoredText overlap with other artists is {overlap(ax, _draw_text_bbox(ax))}."
+        )
+        logging.info("Scaling y-axis by 5% to fit legend")
+        ax.set_ylim(ax.get_ylim()[0], ax.get_ylim()[-1] * scale_factor)
         ax.figure.canvas.draw()
+        if max_scales > 10:
+            if not soft_fail:
+                raise RuntimeError(
+                    "Could not fit AnchoredText in 10 iterations, return anyway by passing `soft_fail=True`."
+                )
+            else:
+                logging.warning("Could not fit AnchoredText in 10 iterations")
+                break
+        max_scales += 1
     return ax
 
 
@@ -711,7 +1103,7 @@ def mpl_magic(ax=None, info=True):
 
     ax = ylow(ax)
     ax = yscale_legend(ax)
-    ax = yscale_text(ax)
+    ax = yscale_anchored_text(ax)
 
     return ax
 
@@ -757,8 +1149,8 @@ class RemainderFixed(axes_size.Scaled):
         self.div = divider
 
     def get_size(self, renderer):
-        xrel, xabs = axes_size.AddList(self.xsizes).get_size(renderer)
-        yrel, yabs = axes_size.AddList(self.ysizes).get_size(renderer)
+        xrel, xabs = sum(self.xsizes, start=axes_size.Fixed(0)).get_size(renderer)
+        yrel, yabs = sum(self.ysizes, start=axes_size.Fixed(0)).get_size(renderer)
         bb = Bbox.from_bounds(*self.div.get_position()).transformed(
             self.div._fig.transFigure
         )
@@ -864,7 +1256,7 @@ def hist_legend(ax=None, **kwargs):
 
     handles, labels = ax.get_legend_handles_labels()
     new_handles = [
-        Line2D([], [], c=h.get_edgecolor()) if type(h) == mpl.patches.Polygon else h
+        Line2D([], [], c=h.get_edgecolor()) if isinstance(h, mpl.patches.Polygon) else h
         for h in handles
     ]
     ax.legend(handles=new_handles[::-1], labels=labels[::-1], **kwargs)
